@@ -20,6 +20,11 @@ with open(private_key_path, "r") as pkey:
     # Set a license
     w.license = "CC"
 
+    # Add a country
+    c = w.add_country("DE")
+    c.continent_code = "EU"
+    c.name = "Germany"
+
     # Add an AS
     a = w.add_as(204867)
     a.name = "Lightning Wire Labs GmbH"
@@ -30,6 +35,7 @@ with open(private_key_path, "r") as pkey:
     n = w.add_network("2a07:1c44:5800::/40")
     n.country_code = "DE"
     n.asn = a.number
+    n.set_flag(location.NETWORK_FLAG_ANYCAST)
 
     print(n)
 
