@@ -248,6 +248,63 @@ static PyObject* Network_get__last_address(NetworkObject* self) {
 	return PyBytes_FromAddress(address);
 }
 
+static PyObject* Network_richcompare(NetworkObject* self, PyObject* other, int op) {
+	int r;
+
+	// Check for type
+	if (!PyObject_IsInstance(other, (PyObject *)&NetworkType))
+		Py_RETURN_NOTIMPLEMENTED;
+
+	NetworkObject* o = (NetworkObject*)other;
+
+	r = loc_network_cmp(self->network, o->network);
+
+	switch (op) {
+		case Py_EQ:
+			if (r == 0)
+				Py_RETURN_TRUE;
+
+			Py_RETURN_FALSE;
+
+		case Py_LT:
+			if (r < 0)
+				Py_RETURN_TRUE;
+
+			Py_RETURN_FALSE;
+
+		default:
+			break;
+	}
+
+	Py_RETURN_NOTIMPLEMENTED;
+}
+
+static PyObject* Network_reverse_pointer(NetworkObject* self, PyObject* args, PyObject* kwargs) {
+	const char* kwlist[] = { "suffix", NULL };
+	const char* suffix = NULL;
+	char* rp = NULL;
+
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|z", (char**)kwlist, &suffix))
+		return NULL;
+
+	rp = loc_network_reverse_pointer(self->network, suffix);
+	if (!rp) {
+		switch (errno) {
+			case ENOTSUP:
+				Py_RETURN_NONE;
+
+			default:
+				PyErr_SetFromErrno(PyExc_OSError);
+				return NULL;
+		}
+	}
+
+	PyObject* ret = PyUnicode_FromString(rp);
+	free(rp);
+
+	return ret;
+}
+
 static struct PyMethodDef Network_methods[] = {
 	{
 		"exclude",
@@ -267,6 +324,12 @@ static struct PyMethodDef Network_methods[] = {
 		METH_VARARGS,
 		NULL,
 	},
+	{
+		"reverse_pointer",
+		(PyCFunction)Network_reverse_pointer,
+		METH_VARARGS|METH_KEYWORDS,
+		NULL,
+	},
 	{
 		"set_flag",
 		(PyCFunction)Network_set_flag,
@@ -342,4 +405,5 @@ PyTypeObject NetworkType = {
 	.tp_getset =             Network_getsetters,
 	.tp_repr =               (reprfunc)Network_repr,
 	.tp_str =                (reprfunc)Network_str,
+	.tp_richcompare =        (richcmpfunc)Network_richcompare,
 };
