@@ -127,7 +127,7 @@ int loc_stringpool_open(struct loc_ctx* ctx, struct loc_stringpool** pool,
 	p->data   = data;
 	p->length = length;
 
-	DEBUG(p->ctx, "Opened string pool at %p (%zu bytes)\n", p->data, p->length);
+	DEBUG(p->ctx, "Opened string pool at %p (%zd bytes)\n", p->data, p->length);
 
 	*pool = p;
 	return 0;
