@@ -40,6 +40,7 @@
 #include <libloc/database.h>
 #include <libloc/format.h>
 #include <libloc/network.h>
+#include <libloc/network-tree.h>
 #include <libloc/private.h>
 #include <libloc/writer.h>
 
@@ -478,7 +479,7 @@ static int loc_database_write_networks(struct loc_writer* writer,
 		}
 
 		// Write the current node
-		DEBUG(writer->ctx, "Writing node %p (0 = %d, 1 = %d)\n",
+		DEBUG(writer->ctx, "Writing node %p (0 = %u, 1 = %u)\n",
 			node, node->index_zero, node->index_one);
 
 		*offset += fwrite(&db_node, 1, sizeof(db_node), f);
@@ -642,6 +643,8 @@ END:
 }
 
 LOC_EXPORT int loc_writer_write(struct loc_writer* writer, FILE* f, enum loc_database_version version) {
+	size_t bytes_written = 0;
+
 	// Check version
 	switch (version) {
 		case LOC_DATABASE_VERSION_UNSET:
@@ -652,11 +655,11 @@ LOC_EXPORT int loc_writer_write(struct loc_writer* writer, FILE* f, enum loc_dat
 			break;
 
 		default:
-			ERROR(writer->ctx, "Invalid database version: %d\n", version);
+			ERROR(writer->ctx, "Invalid database version: %u\n", version);
 			return -1;
 	}
 
-	DEBUG(writer->ctx, "Writing database in version %d\n", version);
+	DEBUG(writer->ctx, "Writing database in version %u\n", version);
 
 	struct loc_database_magic magic;
 	make_magic(writer, &magic, version);
@@ -765,7 +768,16 @@ LOC_EXPORT int loc_writer_write(struct loc_writer* writer, FILE* f, enum loc_dat
 	if (r)
 		return r;
 
-	fwrite(&header, 1, sizeof(header), f);
+	bytes_written = fwrite(&header, 1, sizeof(header), f);
+	if (bytes_written < sizeof(header)) {
+		ERROR(writer->ctx, "Could not write header: %s\n", strerror(errno));
+		return r;
+	}
+
+	// Seek back to the end
+	r = fseek(f, 0, SEEK_END);
+	if (r)
+		return r;
 
 	// Flush everything
 	fflush(f);
